@@ -1,5 +1,4 @@
 open Stdune
-open Dune_rules
 
 module Kind = struct
   type t =
@@ -18,9 +17,9 @@ module Kind = struct
   let lowest_priority = max_int
 
   let of_dir_contents files =
-    if String.Set.mem files Workspace.filename
+    if String.Set.mem files Dune_rules.Workspace.filename
     then Some Dune_workspace
-    else if Filename.Set.mem files Dune_project.filename
+    else if Filename.Set.mem files Source.Dune_project.filename
     then Some Dune_project
     else None
   ;;
@@ -96,22 +95,32 @@ let create ~default_is_cwd ~specified_by_user =
         | None -> if default_is_cwd then Some cwd else None)
   with
   | Some { Candidate.dir; to_cwd; kind } ->
-    { kind
-    ; dir
-    ; to_cwd
-    ; reach_from_root_prefix = String.concat ~sep:"" (List.map to_cwd ~f:(sprintf "%s/"))
-    }
+    Ok
+      { kind
+      ; dir
+      ; to_cwd
+      ; reach_from_root_prefix =
+          String.concat ~sep:"" (List.map to_cwd ~f:(sprintf "%s/"))
+      }
   | None ->
-    User_error.raise
-      [ Pp.text "I cannot find the root of the current workspace/project."
-      ; Pp.text "If you would like to create a new dune project, you can type:"
-      ; Pp.nop
-      ; Pp.verbatim "    dune init project NAME"
-      ; Pp.nop
-      ; Pp.text
-          "Otherwise, please make sure to run dune inside an existing project or \
-           workspace. For more information about how dune identifies the root of the \
-           current workspace/project, please refer to \
-           https://dune.readthedocs.io/en/stable/usage.html#finding-the-root"
-      ]
+    Error
+      User_error.(
+        make
+          [ Pp.text "I cannot find the root of the current workspace/project."
+          ; Pp.text "If you would like to create a new dune project, you can type:"
+          ; Pp.nop
+          ; Pp.verbatim "    dune init project NAME"
+          ; Pp.nop
+          ; Pp.text
+              "Otherwise, please make sure to run dune inside an existing project or \
+               workspace. For more information about how dune identifies the root of the \
+               current workspace/project, please refer to \
+               https://dune.readthedocs.io/en/stable/usage.html#finding-the-root"
+          ])
+;;
+
+let create_exn ~default_is_cwd ~specified_by_user =
+  match create ~default_is_cwd ~specified_by_user with
+  | Ok x -> x
+  | Error e -> raise (User_error.E e)
 ;;
