@@ -2,6 +2,11 @@ module Promote = struct
   type t =
     | Automatically
     | Never
+
+  let to_dyn = function
+    | Automatically -> Dyn.variant "Automatically" []
+    | Never -> Dyn.variant "Never" []
+  ;;
 end
 
 let report_errors_config = ref Report_errors_config.default
