@@ -40,3 +40,9 @@ module Source_kind = Source_kind
 module Package_info = Package_info
 module Section = Section
 module Package = Package
+module Dune_project_name = Dune_project_name
+module Dialect = Dialect
+module Lib_mode = Lib_mode
+module Melange = Melange
+module Module_name = Module_name
+module Preprocess = Preprocess
