@@ -1,11 +1,12 @@
-open Dune_rpc_private
+open Import
+open Dune_rpc
 
-(* Internal RPC requests *)
+(** Internal RPC requests *)
 
-module Build_outcome : sig
-  type t = Dune_engine.Scheduler.Run.Build_outcome.t =
+module Build_outcome_with_diagnostics : sig
+  type t =
     | Success
-    | Failure
+    | Failure of Dune_engine.Compound_user_error.t list
 
   val sexp : (t, Conv.values) Conv.t
 end
@@ -24,6 +25,5 @@ module Status : sig
   val sexp : (t, Conv.values) Conv.t
 end
 
-val build : (string list, Build_outcome.t) Decl.Request.t
-
+val build : (string list, Build_outcome_with_diagnostics.t) Decl.Request.t
 val status : (unit, Status.t) Decl.Request.t
