@@ -17,11 +17,12 @@ include struct
   open Dune_engine
   module Build_config = Build_config
   module Diff_promotion = Diff_promotion
+  module Build_outcome = Scheduler.Run.Build_outcome
 end
 
 include struct
   open Decl
-  module Build_outcome = Build_outcome
+  module Build_outcome_with_diagnostics = Build_outcome_with_diagnostics
   module Status = Status
 end
 
@@ -107,7 +108,8 @@ module Run = struct
   ;;
 end
 
-type 'a pending_build_action = Build of 'a list * Build_outcome.t Fiber.Ivar.t
+type 'a pending_build_action =
+  | Build of 'a list * Dune_engine.Scheduler.Run.Build_outcome.t Fiber.Ivar.t
 
 module Client = Stdune.Unit
 
@@ -192,7 +194,6 @@ type 'a t =
   { config : Run.t
   ; pending_build_jobs : ('a list * Build_outcome.t Fiber.Ivar.t) Job_queue.t
   ; parse_build : string -> 'a
-  ; watch_mode_config : Watch_mode_config.t
   ; mutable clients : Clients.t
   }
 
@@ -208,6 +209,16 @@ let stop (t : _ t) =
   | Some server -> Csexp_rpc.Server.stop server
 ;;
 
+let get_current_diagnostic_errors () =
+  Fiber.Svar.read Build_system.errors
+  |> Build_system_error.Set.current
+  |> Build_system_error.Id.Map.values
+  |> List.filter_map ~f:(fun error ->
+    match Build_system_error.description error with
+    | `Exn _ -> None
+    | `Diagnostic compound_user_error -> Some compound_user_error)
+;;
+
 let handler (t : _ t Fdecl.t) handle : 'a Dune_rpc_server.Handler.t =
   let on_init session (_ : Initialize.Request.t) =
     let t = Fdecl.get t in
@@ -321,23 +332,13 @@ let handler (t : _ t Fdecl.t) handle : 'a Dune_rpc_server.Handler.t =
   let () =
     let build _session targets =
       let server = Fdecl.get t in
-      match server.watch_mode_config with
-      | No -> assert false
-      | Yes Eager ->
-        let error =
-          Dune_rpc.Response.Error.create
-            ~kind:Invalid_request
-            ~message:
-              "the rpc server is running with eager watch mode using --watch. to run \
-               builds through an rpc client, start the server using --passive-watch-mode"
-            ()
-        in
-        raise (Dune_rpc.Response.Error.E error)
-      | Yes Passive ->
-        let ivar = Fiber.Ivar.create () in
-        let targets = List.map targets ~f:server.parse_build in
-        let* () = Job_queue.write server.pending_build_jobs (targets, ivar) in
-        Fiber.Ivar.read ivar
+      let ivar = Fiber.Ivar.create () in
+      let targets = List.map targets ~f:server.parse_build in
+      let* () = Job_queue.write server.pending_build_jobs (targets, ivar) in
+      let+ build_outcome = Fiber.Ivar.read ivar in
+      match (build_outcome : Build_outcome.t) with
+      | Success -> Build_outcome_with_diagnostics.Success
+      | Failure -> Failure (get_current_diagnostic_errors ())
     in
     Handler.implement_request rpc Decl.build build
   in
@@ -405,7 +406,7 @@ let handler (t : _ t Fdecl.t) handle : 'a Dune_rpc_server.Handler.t =
   rpc
 ;;
 
-let create ~lock_timeout ~registry ~root ~watch_mode_config ~handle stats ~parse_build =
+let create ~lock_timeout ~registry ~root ~handle stats ~parse_build =
   let t = Fdecl.create Dyn.opaque in
   let pending_build_jobs = Job_queue.create () in
   let handler = Dune_rpc_server.make (handler t handle) in
@@ -443,14 +444,7 @@ let create ~lock_timeout ~registry ~root ~watch_mode_config ~handle stats ~parse
     ; server_ivar = Fiber.Ivar.create ()
     }
   in
-  let res =
-    { config
-    ; pending_build_jobs
-    ; watch_mode_config
-    ; clients = Clients.empty
-    ; parse_build
-    }
-  in
+  let res = { config; pending_build_jobs; clients = Clients.empty; parse_build } in
   Fdecl.set t res;
   res
 ;;
