@@ -130,20 +130,20 @@ module Builder = struct
   ;;
 
   let set_workspace_base
-    t
-    { Workspace.Context.Common.targets = _
-    ; name
-    ; host_context = _
-    ; profile
-    ; env = _
-    ; toolchain
-    ; paths
-    ; loc = _
-    ; fdo_target_exe
-    ; dynamically_linked_foreign_archives
-    ; instrument_with
-    ; merlin
-    }
+        t
+        { Workspace.Context.Common.targets = _
+        ; name
+        ; host_context = _
+        ; profile
+        ; env = _
+        ; toolchain
+        ; paths
+        ; loc = _
+        ; fdo_target_exe
+        ; dynamically_linked_foreign_archives
+        ; instrument_with
+        ; merlin
+        }
     =
     let env =
       let env = Global.env () in
@@ -400,68 +400,97 @@ let create (builder : Builder.t) ~(kind : Kind.t) =
     | Default | Opam _ -> builder
     | Lock _ ->
       let env =
-        Memo.lazy_ (fun () ->
-          let+ current_env = builder.env
-          and+ pkg_env = Pkg_rules.exported_env builder.name in
-          Env_path.extend_env_concat_path current_env pkg_env)
+        Memo.lazy_
+          ~human_readable_description:(fun () ->
+            Pp.textf
+              "base environment for context %S"
+              (Context_name.to_string builder.name))
+          (fun () ->
+             let+ current_env = builder.env
+             and+ pkg_env = Pkg_rules.exported_env builder.name in
+             Env_path.extend_env_concat_path current_env pkg_env)
         |> Memo.Lazy.force
       in
       { builder with env }
   in
+  let which_outside_lockdir = Which.which ~path:builder.path in
   let which =
     match kind with
-    | Default | Opam _ -> Which.which ~path:builder.path
+    | Default | Opam _ -> which_outside_lockdir
     | Lock _ ->
       let which = Staged.unstage @@ Pkg_rules.which builder.name in
       fun prog ->
-        which prog
-        >>= (function
-         | Some p -> Memo.return (Some p)
-         | None -> Which.which ~path:builder.path prog)
+        Memo.push_stack_frame
+          ~human_readable_description:(fun () ->
+            Pp.textf
+              "looking up binary %S in context %S"
+              prog
+              (Context_name.to_string builder.name))
+          (fun () ->
+             which prog
+             >>= function
+             | Some p -> Memo.return (Some p)
+             | None -> Which.which ~path:builder.path prog)
   in
   let ocamlpath =
-    Memo.lazy_ (fun () ->
-      match kind with
-      | Lock _ -> Pkg_rules.ocamlpath builder.name
-      | Default | Opam _ ->
-        let+ ocamlpath = builder.env >>| Findlib_config.ocamlpath_of_env in
-        Kind.ocamlpath kind ~ocamlpath ~findlib_toolchain:builder.findlib_toolchain)
+    Memo.lazy_
+      ~human_readable_description:(fun () ->
+        Pp.textf "loading OCAMLPATH for context %S" (Context_name.to_string builder.name))
+      (fun () ->
+         match kind with
+         | Lock _ -> Pkg_rules.ocamlpath builder.name
+         | Default | Opam _ ->
+           let+ ocamlpath = builder.env >>| Findlib_config.ocamlpath_of_env in
+           Kind.ocamlpath kind ~ocamlpath ~findlib_toolchain:builder.findlib_toolchain)
   in
   let findlib =
-    Memo.lazy_ (fun () ->
-      let ocamlpath = Memo.Lazy.force ocamlpath in
-      let* env = builder.env in
-      let findlib_toolchain =
-        Option.map builder.findlib_toolchain ~f:Context_name.to_string
-      in
-      Findlib_config.discover_from_env ~env ~which ~ocamlpath ~findlib_toolchain)
+    Memo.lazy_
+      ~human_readable_description:(fun () ->
+        Pp.textf "loading findlib for context %S" (Context_name.to_string builder.name))
+      (fun () ->
+         let ocamlpath = Memo.Lazy.force ocamlpath in
+         let* env = builder.env in
+         let findlib_toolchain =
+           Option.map builder.findlib_toolchain ~f:Context_name.to_string
+         in
+         Findlib_config.discover_from_env ~env ~which ~ocamlpath ~findlib_toolchain)
   in
   let ocaml_and_build_env_kind =
-    Memo.Lazy.create ~name:"ocaml_and_build_env_kind" (fun () ->
-      let+ ocaml, env =
-        let* findlib = Memo.Lazy.force findlib
-        and* env = builder.env in
-        let toolchain kind =
-          let+ toolchain =
-            Ocaml_toolchain.of_env_with_findlib builder.name env findlib ~which
-          in
-          toolchain, kind
-        in
-        match kind with
-        | Default -> toolchain `Default
-        | Opam _ -> toolchain `Opam
-        | Lock _ ->
-          Pkg_rules.ocaml_toolchain builder.name
-          >>= (function
-           | None -> toolchain `Lock
-           | Some toolchain ->
-             let+ toolchain, _ = Action_builder.evaluate_and_collect_facts toolchain in
-             toolchain, `Default)
-      in
-      Ocaml_toolchain.register_response_file_support ocaml;
-      if Option.is_some builder.fdo_target_exe
-      then Ocaml_toolchain.check_fdo_support ocaml builder.name;
-      ocaml, env)
+    Memo.Lazy.create
+      ~name:"ocaml_and_build_env_kind"
+      ~human_readable_description:(fun () ->
+        Pp.textf
+          "loading the OCaml compiler for context %S"
+          (Context_name.to_string builder.name))
+      (fun () ->
+         let+ ocaml, env =
+           let* findlib = Memo.Lazy.force findlib
+           and* env = builder.env in
+           let toolchain kind =
+             let+ toolchain =
+               Ocaml_toolchain.of_env_with_findlib
+                 builder.name
+                 env
+                 findlib
+                 ~which:which_outside_lockdir
+             in
+             toolchain, kind
+           in
+           match kind with
+           | Default -> toolchain `Default
+           | Opam _ -> toolchain `Opam
+           | Lock _ ->
+             Pkg_rules.ocaml_toolchain builder.name
+             >>= (function
+              | None -> toolchain `Lock
+              | Some toolchain ->
+                let+ toolchain, _ = Action_builder.evaluate_and_collect_facts toolchain in
+                toolchain, `Lock)
+         in
+         Ocaml_toolchain.register_response_file_support ocaml;
+         if Option.is_some builder.fdo_target_exe
+         then Ocaml_toolchain.check_fdo_support ocaml builder.name;
+         ocaml, env)
   in
   let default_ocamlpath =
     Memo.Lazy.create ~name:"default_ocamlpath" ~cutoff:(List.equal Path.equal) (fun () ->
@@ -481,10 +510,15 @@ let create (builder : Builder.t) ~(kind : Kind.t) =
   in
   let builder =
     let installed_env =
-      Memo.lazy_ (fun () ->
-        let* findlib = Memo.Lazy.force findlib in
-        let+ env = builder.env in
-        make_installed_env env builder.name findlib builder.env_nodes builder.profile)
+      Memo.lazy_
+        ~human_readable_description:(fun () ->
+          Pp.textf
+            "creating installed environment for %S"
+            (Context_name.to_string builder.name))
+        (fun () ->
+           let* findlib = Memo.Lazy.force findlib in
+           let+ env = builder.env in
+           make_installed_env env builder.name findlib builder.env_nodes builder.profile)
     in
     { builder with env = Memo.Lazy.force installed_env }
   in
@@ -678,8 +712,8 @@ module DB = struct
         "context-db-get"
         ~input:(module Context_name)
         (fun name ->
-          let+ contexts = all () in
-          List.find_exn contexts ~f:(fun c -> Context_name.equal name c.builder.name))
+           let+ contexts = all () in
+           List.find_exn contexts ~f:(fun c -> Context_name.equal name c.builder.name))
     in
     Memo.exec memo
   ;;
@@ -709,6 +743,19 @@ let map_exe (context : t) =
        | _ -> exe)
 ;;
 
+let roots =
+  lazy
+    (let open Setup in
+     match prefix with
+     | None -> roots
+     | Some prefix ->
+       let prefix = Install.Roots.make prefix ~relative:Filename.concat in
+       Install.Roots.map2 roots prefix ~f:(fun root prefix ->
+         match root with
+         | None -> Some prefix
+         | Some _ -> root))
+;;
+
 let roots t =
   let module Roots = Install.Roots in
   let+ prefix_roots =
@@ -722,7 +769,7 @@ let roots t =
   in
   match t.kind with
   | Lock _ | Default ->
-    let setup_roots = Roots.map ~f:(Option.map ~f:Path.of_string) Setup.roots in
+    let setup_roots = Roots.map ~f:(Option.map ~f:Path.of_string) (Lazy.force roots) in
     Roots.first_has_priority setup_roots prefix_roots
   | Opam _ -> prefix_roots
 ;;
