@@ -1,5 +1,4 @@
-open! Dune_engine
-open Stdune
+open Import
 
 (** The default environment in which to run commands. The environment of
     individual build contexts augment this environment. *)
