@@ -1,5 +1,68 @@
 open Import
 open Memo.O
+module Mode_conf = Inline_tests_info.Mode_conf
+
+let action
+      sctx
+      ~deps
+      ~loc
+      ~dir
+      ~inline_test_dir
+      ~runner_name
+      (mode : Mode_conf.t)
+      (flags : string list Action_builder.t)
+  : Action.t Action_builder.t
+  =
+  let exe =
+    let ext =
+      match mode with
+      | Native | Best -> ".exe"
+      | Jsoo mode -> Js_of_ocaml.Ext.exe ~mode
+      | Byte -> ".bc"
+    in
+    Path.build (Path.Build.relative inline_test_dir (runner_name ^ ext))
+  in
+  let open Action_builder.O in
+  let+ action =
+    (* [action] needs to run from [dir] as we use [dir] to resolve
+       the exe path in case of a custom [runner] *)
+    match
+      match mode with
+      | Native | Best | Byte -> None
+      | Jsoo _ -> Some Jsoo_rules.runner
+    with
+    | None -> flags >>| Action.run (Ok exe)
+    | Some runner ->
+      let* prog =
+        Super_context.resolve_program
+          ~dir
+          sctx
+          ~where:Original_path
+          ~loc:(Some loc)
+          runner
+      and* flags = flags in
+      let action =
+        Path.reach exe ~from:(Path.build dir) :: flags
+        |> Action.run prog
+        |> Action_builder.return
+      in
+      (* jeremiedimino: it feels like this pattern should be pushed
+         into [resolve_program] directly *)
+      (match prog with
+       | Error _ -> action
+       | Ok p -> Action_builder.path p >>> action)
+  and+ () = deps
+  and+ () = Action_builder.path exe
+  and+ () =
+    match mode with
+    | Native | Best | Byte | Jsoo JS -> Action_builder.return ()
+    | Jsoo Wasm ->
+      Path.Build.relative inline_test_dir (runner_name ^ Js_of_ocaml.Ext.wasm_dir)
+      |> Path.build
+      |> Action_builder.path
+  in
+  Action.chdir (Path.build dir) action
+;;
 
 module Backend = struct
   module M = struct
@@ -63,52 +126,86 @@ module Backend = struct
   include Sub_system.Register_backend (M)
 end
 
+module Info = Inline_tests_info.Tests
+
+let flags ~(info : Info.t) ~expander ~backends ~lib_name ~partition =
+  let expander =
+    let bindings =
+      let bindings =
+        Pform.Map.singleton
+          (Pform.Var Library_name)
+          [ Value.String (Lib_name.Local.to_string lib_name) ]
+      in
+      match partition with
+      | None -> bindings
+      | Some p -> Pform.Map.add_exn bindings (Pform.Var Partition) [ Value.String p ]
+    in
+    Expander.add_bindings expander ~bindings
+  in
+  let open Action_builder.O in
+  List.map backends ~f:(fun (backend : Backend.t) -> backend.info.flags) @ [ info.flags ]
+  |> List.map
+       ~f:(Expander.expand_and_eval_set expander ~standard:(Action_builder.return []))
+  |> Action_builder.all
+  >>| List.concat
+;;
+
+let partition_flags ~expander ~lib_name ~(backends : Backend.t list) =
+  match
+    List.filter_map backends ~f:(fun backend -> backend.info.list_partitions_flags)
+  with
+  | [] -> None
+  | flags ->
+    let flags =
+      let open Action_builder.O in
+      let expander =
+        let bindings =
+          Pform.Map.singleton
+            (Pform.Var Library_name)
+            [ Value.String (Lib_name.Local.to_string lib_name) ]
+        in
+        Expander.add_bindings expander ~bindings
+      in
+      List.map
+        flags
+        ~f:(Expander.expand_and_eval_set expander ~standard:(Action_builder.return []))
+      |> Action_builder.all
+      >>| List.concat
+    in
+    Some flags
+;;
+
 include Sub_system.Register_end_point (struct
     module Backend = Backend
-    module Mode_conf = Inline_tests_info.Mode_conf
     module Info = Inline_tests_info.Tests
 
-    let gen_rules c ~expander ~(info : Info.t) ~backends =
-      let { Sub_system.Library_compilation_context.super_context = sctx
+    let gen_rules
+          { Sub_system.Library_compilation_context.super_context = sctx
           ; dir
           ; stanza = lib
           ; scope
           ; source_modules
           ; compile_info = _
           }
-        =
-        c
-      in
+          ~expander
+          ~(info : Info.t)
+          ~backends
+      =
       let loc = lib.buildable.loc in
-      let lib_name = snd lib.name in
       let inline_test_dir =
+        let lib_name = snd lib.name in
         Path.Build.relative dir (Inline_tests_info.inline_test_dirname lib_name)
       in
       let runner_name = Inline_tests_info.inline_test_runner in
-      let obj_dir = Obj_dir.make_exe ~dir:inline_test_dir ~name:"t" in
       let main_module =
         let name = Module_name.of_string "main" in
         Module.generated ~kind:Impl ~src_dir:inline_test_dir [ name ]
       in
-      let modules = Modules.With_vlib.singleton_exe main_module in
-      let runner_libs =
-        let open Resolve.Memo.O in
-        let* libs =
-          Resolve.Memo.List.concat_map backends ~f:(fun (backend : Backend.t) ->
-            backend.runner_libraries)
-        in
-        let* lib = Lib.DB.resolve (Scope.libs scope) (loc, Library.best_name lib) in
-        let* more_libs =
-          Resolve.Memo.List.map info.libraries ~f:(Lib.DB.resolve (Scope.libs scope))
-        in
-        Lib.closure ~linking:true ((lib :: libs) @ more_libs)
-      in
       (* Generate the runner file *)
       let js_of_ocaml =
-        Js_of_ocaml.Mode.Pair.map
-          ~f:(fun (x : Js_of_ocaml.In_context.t) ->
-            { x with javascript_files = []; wasm_files = [] })
-          (Js_of_ocaml.In_context.make ~dir lib.buildable.js_of_ocaml)
+        Js_of_ocaml.In_context.make ~dir lib.buildable.js_of_ocaml
+        |> Js_of_ocaml.Mode.Pair.map ~f:(fun (x : Js_of_ocaml.In_context.t) ->
+          { x with javascript_files = []; wasm_files = [] })
       in
       let* () =
         Super_context.add_rule
@@ -120,17 +217,20 @@ include Sub_system.Register_end_point (struct
              |> Option.value_exn
              |> Path.as_in_build_dir_exn
            in
-           let files ml_kind =
-             Value.L.paths (List.filter_map source_modules ~f:(Module.file ~ml_kind))
-           in
-           let bindings =
-             Pform.Map.of_list_exn
-               [ Var Impl_files, files Impl; Var Intf_files, files Intf ]
-           in
-           let expander = Expander.add_bindings expander ~bindings in
            let action =
              let open Action_builder.With_targets.O in
              let+ actions =
+               let expander =
+                 let bindings =
+                   let files ml_kind =
+                     List.filter_map source_modules ~f:(Module.file ~ml_kind)
+                     |> Value.L.paths
+                   in
+                   Pform.Map.of_list_exn
+                     [ Var Impl_files, files Impl; Var Intf_files, files Intf ]
+                 in
+                 Expander.add_bindings expander ~bindings
+               in
                List.filter_map backends ~f:(fun (backend : Backend.t) ->
                  Option.map backend.info.generate_runner ~f:(fun (loc, action) ->
                    Action_unexpanded.expand_no_targets
@@ -149,10 +249,27 @@ include Sub_system.Register_end_point (struct
            Action_builder.With_targets.add ~file_targets:[ target ] action)
       and* cctx =
         let package = Library.package lib in
-        let* ocaml_flags =
-          Buildable_rules.ocaml_flags sctx ~dir info.executable_ocaml_flags
+        let* flags =
+          let+ ocaml_flags =
+            Buildable_rules.ocaml_flags sctx ~dir info.executable_ocaml_flags
+          in
+          Ocaml_flags.append_common ocaml_flags [ "-w"; "-24"; "-g" ]
+        in
+        let obj_dir = Obj_dir.make_exe ~dir:inline_test_dir ~name:"t" in
+        let modules = Modules.With_vlib.singleton_exe main_module in
+        let runner_libs =
+          let lib_db = Scope.libs scope in
+          let open Resolve.Memo.O in
+          let* libs =
+            Resolve.Memo.List.concat_map backends ~f:(fun (backend : Backend.t) ->
+              backend.runner_libraries)
+          in
+          let* lib = Lib.DB.resolve lib_db (loc, Library.best_name lib) in
+          let* more_libs =
+            Resolve.Memo.List.map info.libraries ~f:(Lib.DB.resolve lib_db)
+          in
+          Lib.closure ~linking:true ((lib :: libs) @ more_libs)
         in
-        let flags = Ocaml_flags.append_common ocaml_flags [ "-w"; "-24"; "-g" ] in
         Compilation_context.create
           ()
           ~super_context:sctx
@@ -177,9 +294,23 @@ include Sub_system.Register_end_point (struct
           | Native | Best | Byte -> true
           | Jsoo mode -> Js_of_ocaml.Mode.Pair.select ~mode jsoo_enabled_modes)
       in
-      let* linkages =
-        let ocaml = Compilation_context.ocaml cctx in
-        let l =
+      let* (_ : Exe.dep_graphs) =
+        let* linkages =
+          let+ jsoo =
+            let+ jsoo_is_whole_program =
+              Jsoo_rules.jsoo_is_whole_program sctx ~dir ~in_context:js_of_ocaml
+            in
+            if
+              List.exists modes ~f:(fun mode ->
+                match (mode : Mode_conf.t) with
+                | Jsoo mode -> Js_of_ocaml.Mode.Pair.select ~mode jsoo_is_whole_program
+                | Native | Best | Byte -> false)
+            then [ Exe.Linkage.byte_for_jsoo ]
+            else []
+          in
+          jsoo
+          @
+          let ocaml = Compilation_context.ocaml cctx in
           List.map modes ~f:(fun (mode : Mode_conf.t) ->
             match mode with
             | Native -> Exe.Linkage.native
@@ -188,18 +319,6 @@ include Sub_system.Register_end_point (struct
             | Jsoo JS -> Exe.Linkage.js
             | Jsoo Wasm -> Exe.Linkage.wasm)
         in
-        let+ jsoo_is_whole_program =
-          Jsoo_rules.jsoo_is_whole_program sctx ~dir ~in_context:js_of_ocaml
-        in
-        if
-          List.exists modes ~f:(fun mode ->
-            match (mode : Mode_conf.t) with
-            | Jsoo mode -> Js_of_ocaml.Mode.Pair.select ~mode jsoo_is_whole_program
-            | Native | Best | Byte -> false)
-        then Exe.Linkage.byte_for_jsoo :: l
-        else l
-      in
-      let* (_ : Exe.dep_graphs) =
         let link_args =
           let open Action_builder.O in
           let+ link_args_info =
@@ -217,123 +336,19 @@ include Sub_system.Register_end_point (struct
           ~link_args
           ~promote:None
       in
-      let partitions_flags : string list Action_builder.t option =
-        match
-          List.filter_map backends ~f:(fun backend -> backend.info.list_partitions_flags)
-        with
-        | [] -> None
-        | flags ->
-          let flags =
-            let open Action_builder.O in
-            let+ l =
-              let expander =
-                let bindings =
-                  Pform.Map.singleton
-                    (Pform.Var Library_name)
-                    [ Value.String (Lib_name.Local.to_string (snd lib.name)) ]
-                in
-                Expander.add_bindings expander ~bindings
-              in
-              List.map
-                flags
-                ~f:
-                  (Expander.expand_and_eval_set
-                     expander
-                     ~standard:(Action_builder.return []))
-              |> Action_builder.all
-            in
-            List.concat l
-          in
-          Some flags
+      let partitions_flags =
+        partition_flags ~expander ~lib_name:(snd lib.name) ~backends
       in
-      let sandbox =
-        let project = Scope.project scope in
-        if Dune_project.dune_version project < (3, 5)
-        then Sandbox_config.no_special_requirements
-        else Sandbox_config.needs_sandboxing
+      let deps, sandbox =
+        let sandbox =
+          let project = Scope.project scope in
+          if Dune_project.dune_version project < (3, 5)
+          then Sandbox_config.no_special_requirements
+          else Sandbox_config.needs_sandboxing
+        in
+        Dep_conf_eval.unnamed ~sandbox info.deps ~expander
       in
-      let deps, sandbox = Dep_conf_eval.unnamed ~sandbox info.deps ~expander in
-      let action (mode : Mode_conf.t) (flags : string list Action_builder.t)
-        : Action.t Action_builder.t
-        =
-        (* [action] needs to run from [dir] as we use [dir] to resolve
-           the exe path in case of a custom [runner] *)
-        let ext =
-          match mode with
-          | Native | Best -> ".exe"
-          | Jsoo mode -> Js_of_ocaml.Ext.exe ~mode
-          | Byte -> ".bc"
-        in
-        let custom_runner =
-          match mode with
-          | Native | Best | Byte -> None
-          | Jsoo _ -> Some Jsoo_rules.runner
-        in
-        let exe = Path.build (Path.Build.relative inline_test_dir (runner_name ^ ext)) in
-        let open Action_builder.O in
-        let+ action =
-          match custom_runner with
-          | None ->
-            let+ flags = flags in
-            Action.run (Ok exe) flags
-          | Some runner ->
-            let* prog =
-              Super_context.resolve_program
-                ~dir
-                sctx
-                ~where:Original_path
-                ~loc:(Some loc)
-                runner
-            and* flags = flags in
-            let action =
-              Action.run prog (Path.reach exe ~from:(Path.build dir) :: flags)
-            in
-            (* jeremiedimino: it feels like this pattern should be pushed
-               into [resolve_program] directly *)
-            (match prog with
-             | Error _ -> Action_builder.return action
-             | Ok p -> Action_builder.path p >>> Action_builder.return action)
-        and+ () = deps
-        and+ () = Action_builder.path exe
-        and+ () =
-          match mode with
-          | Native | Best | Byte | Jsoo JS -> Action_builder.return ()
-          | Jsoo Wasm ->
-            Action_builder.path
-              (Path.build
-                 (Path.Build.relative
-                    inline_test_dir
-                    (runner_name ^ Js_of_ocaml.Ext.wasm_dir)))
-        in
-        Action.chdir (Path.build dir) action
-      in
-      let flags partition : string list Action_builder.t =
-        let flags =
-          List.map backends ~f:(fun (backend : Backend.t) -> backend.info.flags)
-          @ [ info.flags ]
-        in
-        let bindings =
-          Pform.Map.singleton
-            (Pform.Var Library_name)
-            [ Value.String (Lib_name.Local.to_string (snd lib.name)) ]
-        in
-        let bindings =
-          match partition with
-          | None -> bindings
-          | Some p -> Pform.Map.add_exn bindings (Pform.Var Partition) [ Value.String p ]
-        in
-        let expander = Expander.add_bindings expander ~bindings in
-        let open Action_builder.O in
-        let+ l =
-          List.map
-            flags
-            ~f:
-              (Expander.expand_and_eval_set expander ~standard:(Action_builder.return []))
-          |> Action_builder.all
-        in
-        List.concat l
-      in
-      let source_files = List.concat_map source_modules ~f:Module.sources in
+      let action = action sctx ~deps ~loc ~dir ~inline_test_dir ~runner_name in
       Memo.parallel_iter modes ~f:(fun (mode : Mode_conf.t) ->
         let partition_file =
           Path.Build.relative inline_test_dir ("partitions-" ^ Mode_conf.to_string mode)
@@ -359,16 +374,18 @@ include Sub_system.Register_end_point (struct
           ~loc:info.loc
           (Alias.make ~dir runtest_alias)
           (let open Action_builder.O in
+           let source_files = List.concat_map source_modules ~f:Module.sources in
            let+ actions =
-             let* partitions_flags =
-               match partitions_flags with
-               | None -> Action_builder.return [ None ]
-               | Some _ ->
-                 let+ partitions = Action_builder.lines_of (Path.build partition_file) in
-                 List.map ~f:(fun x -> Some x) partitions
-             in
-             List.map partitions_flags ~f:(fun p -> action mode (flags p))
-             |> Action_builder.all
+             (match partitions_flags with
+              | None -> Action_builder.return [ None ]
+              | Some _ ->
+                Path.build partition_file
+                |> Action_builder.lines_of
+                >>| List.map ~f:(fun x -> Some x))
+             >>| List.map ~f:(fun partition ->
+               flags ~info ~expander ~backends ~lib_name:(snd lib.name) ~partition
+               |> action mode)
+             >>= Action_builder.all
            and+ () = Action_builder.paths source_files in
            match actions with
            | [] -> Action.Full.empty
@@ -384,15 +401,18 @@ include Sub_system.Register_end_point (struct
              Action.Full.make ~sandbox @@ Action.progn [ run_tests; diffs ]))
     ;;
 
-    let gen_rules c ~(info : Info.t) ~backends =
-      let { dir; Sub_system.Library_compilation_context.super_context = sctx; _ } = c in
+    let gen_rules
+          ({ dir; Sub_system.Library_compilation_context.super_context = sctx; _ } as c)
+          ~(info : Info.t)
+          ~backends
+      =
       let* expander = Super_context.expander sctx ~dir in
-      let* enabled_if = Expander.eval_blang expander info.enabled_if in
-      if enabled_if
-      then gen_rules c ~expander ~info ~backends
-      else (
+      Expander.eval_blang expander info.enabled_if
+      >>= function
+      | true -> gen_rules c ~expander ~info ~backends
+      | false ->
         let alias = Alias.make Alias0.runtest ~dir in
-        Simple_rules.Alias_rules.add_empty sctx ~alias ~loc:info.loc)
+        Simple_rules.Alias_rules.add_empty sctx ~alias ~loc:info.loc
     ;;
   end)
 
