@@ -1,12 +1,11 @@
-open! Dune_engine
-open! Stdune
-open! Import
+open Import
 
 module Backend : sig
   type t =
     { loc : Loc.t
     ; runner_libraries : (Loc.t * Lib_name.t) list
     ; flags : Ordered_set_lang.Unexpanded.t
+    ; list_partitions_flags : Ordered_set_lang.Unexpanded.t option
     ; generate_runner : (Loc.t * Action_unexpanded.t) option
     ; extends : (Loc.t * Lib_name.t) list
     }
@@ -17,15 +16,14 @@ end
 module Mode_conf : sig
   type t =
     | Byte
-    | Javascript
+    | Jsoo of Js_of_ocaml.Mode.t
     | Native
     | Best
 
   val compare : t -> t -> ordering
-
   val to_dyn : t -> Dyn.t
-
   val decode : t Dune_lang.Decoder.t
+  val to_string : t -> string
 
   module Map : Map.S with type key = t
 
@@ -33,7 +31,6 @@ module Mode_conf : sig
     include Set.S with type elt = t and type 'a map = 'a Map.t
 
     val decode : t Dune_lang.Decoder.t
-
     val default : t
   end
 end
@@ -55,3 +52,6 @@ module Tests : sig
 
   include Sub_system_info.S with type t := t
 end
+
+val inline_test_dirname : Lib_name.Local.t -> string
+val inline_test_runner : string
