@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 (* Arguments for the compiler to prevent it from being too clever.
 
@@ -38,13 +39,13 @@ let copy_interface ~sctx ~dir ~obj_dir ~cm_kind m =
     (Module.visibility m <> Visibility.Private
      && Obj_dir.need_dedicated_public_dir obj_dir)
     (fun () ->
-      let cmi_kind = Lib_mode.Cm_kind.cmi cm_kind in
-      Super_context.add_rule
-        sctx
-        ~dir
-        (Action_builder.symlink
-           ~src:(Path.build (Obj_dir.Module.cm_file_exn obj_dir m ~kind:cmi_kind))
-           ~dst:(Obj_dir.Module.cm_public_file_exn obj_dir m ~kind:cmi_kind)))
+       let cmi_kind = Lib_mode.Cm_kind.cmi cm_kind in
+       Super_context.add_rule
+         sctx
+         ~dir
+         (Action_builder.symlink
+            ~src:(Path.build (Obj_dir.Module.cm_file_exn obj_dir m ~kind:cmi_kind))
+            ~dst:(Obj_dir.Module.cm_public_file_exn obj_dir m ~kind:cmi_kind)))
 ;;
 
 let melange_args (cctx : Compilation_context.t) (cm_kind : Lib_mode.Cm_kind.t) module_ =
@@ -83,12 +84,12 @@ let melange_args (cctx : Compilation_context.t) (cm_kind : Lib_mode.Cm_kind.t) m
 ;;
 
 let build_cm
-  cctx
-  ~force_write_cmi
-  ~precompiled_cmi
-  ~cm_kind
-  (m : Module.t)
-  ~(phase : Fdo.phase option)
+      cctx
+      ~force_write_cmi
+      ~precompiled_cmi
+      ~cm_kind
+      (m : Module.t)
+      ~(phase : Fdo.phase option)
   =
   if force_write_cmi && precompiled_cmi
   then Code_error.raise "force_read_cmi and precompiled_cmi are mutually exclusive" [];
@@ -106,7 +107,6 @@ let build_cm
       Sandbox_config.needs_sandboxing
     | _ -> default
   in
-  let open Memo.O in
   let ocaml = Compilation_context.ocaml cctx in
   let* compiler =
     match mode with
@@ -311,21 +311,24 @@ let build_module ?(force_write_cmi = false) ?(precompiled_cmi = false) cctx m =
       match Obj_dir.Module.cm_file obj_dir m ~kind:(Ocaml Cmo) with
       | None -> Memo.return ()
       | Some src ->
-        Compilation_context.js_of_ocaml cctx
-        |> Memo.Option.iter ~f:(fun in_context ->
-          (* Build *.cmo.js *)
-          let sctx = Compilation_context.super_context cctx in
-          let dir = Compilation_context.dir cctx in
-          let action_with_targets =
-            Jsoo_rules.build_cm
-              sctx
-              ~dir
-              ~in_context
-              ~src:(Path.build src)
-              ~obj_dir
-              ~config:None
-          in
-          Super_context.add_rule sctx ~dir action_with_targets))
+        Memo.parallel_iter Js_of_ocaml.Mode.all ~f:(fun mode ->
+          Compilation_context.js_of_ocaml cctx
+          |> Js_of_ocaml.Mode.Pair.select ~mode
+          |> Memo.Option.iter ~f:(fun in_context ->
+            (* Build *.cmo.js / *.wasmo *)
+            let sctx = Compilation_context.super_context cctx in
+            let dir = Compilation_context.dir cctx in
+            let action_with_targets =
+              Jsoo_rules.build_cm
+                sctx
+                ~dir
+                ~in_context
+                ~mode
+                ~src:(Path.build src)
+                ~obj_dir
+                ~config:None
+            in
+            Super_context.add_rule sctx ~dir action_with_targets)))
   in
   Memo.when_ melange (fun () ->
     let* () = build_cm ~cm_kind:(Melange Cmj) ~phase:None in
@@ -456,7 +459,6 @@ let build_alias_module cctx group =
   let sctx = Compilation_context.super_context cctx in
   let file = Option.value_exn (Module.file alias_module ~ml_kind:Impl) in
   let dir = Compilation_context.dir cctx in
-  let open Memo.O in
   let* () =
     Super_context.add_rule
       ~loc:Loc.none
@@ -486,7 +488,6 @@ let build_root_module cctx root_module =
   let sctx = Compilation_context.super_context cctx in
   let file = Option.value_exn (Module.file root_module ~ml_kind:Impl) in
   let dir = Compilation_context.dir cctx in
-  let open Memo.O in
   let* () =
     Super_context.add_rule
       ~loc:Loc.none
@@ -543,7 +544,6 @@ let with_empty_intf ~sctx ~dir module_ =
       (Path.as_in_build_dir_exn name)
       "(* Auto-generated by Dune *)"
   in
-  let open Memo.O in
   let+ () = Super_context.add_rule sctx ~dir rule in
   Module.add_file module_ Ml_kind.Intf (Module.File.make Dialect.ocaml name)
 ;;
