@@ -123,18 +123,31 @@ let user_rule sctx ?extra_bindings ~dir ~expander (rule : Rule_conf.t) =
      | Aliases_with_targets (aliases, alias_target) ->
        let+ () =
          Memo.parallel_iter aliases ~f:(fun alias ->
+           let loc =
+             (* standard aliases don't have a loc *)
+             match Alias0.is_standard alias with
+             | true -> Loc.none
+             | false -> rule.loc
+           in
            let alias = Alias.make ~dir alias in
            Rules.Produce.Alias.add_deps
              alias
+             ~loc
              (Action_builder.path (Path.build alias_target)))
        and+ targets = add_user_rule sctx ~dir ~rule ~action ~expander in
        Some targets
      | Aliases_only aliases ->
        let+ () =
-         let action = interpret_and_add_locks ~expander rule.locks action.build in
-         Memo.parallel_iter aliases ~f:(fun alias ->
-           let alias = Alias.make ~dir alias in
-           Alias_rules.add sctx ~alias ~loc:rule.loc action)
+         match List.map ~f:(Alias.make ~dir) aliases with
+         | [] -> Code_error.raise "empty list of aliases" []
+         | alias :: extra_aliases ->
+           let loc = rule.loc in
+           interpret_and_add_locks ~expander rule.locks action.build
+           |> Alias_rules.add sctx ~alias ~loc
+           >>> Memo.parallel_iter extra_aliases ~f:(fun extra_alias ->
+             Dep.alias alias
+             |> Action_builder.dep
+             |> Rules.Produce.Alias.add_deps ~loc extra_alias)
        in
        None)
 ;;
