@@ -68,7 +68,7 @@ let lock_dir_encode_decode_round_trip_test ?commit ~lock_dir_path ~lock_dir () =
   Lock_dir.Write_disk.(
     prepare ~lock_dir_path ~files:Package_name.Map.empty lock_dir |> commit);
   let lock_dir_round_tripped =
-    try Lock_dir.read_disk lock_dir_path with
+    try Lock_dir.read_disk_exn lock_dir_path with
     | User_error.E _ as exn ->
       let metadata_path =
         Path.Source.relative lock_dir_path Lock_dir.metadata_filename |> Path.source
@@ -130,8 +130,15 @@ let empty_package name ~version =
   { Lock_dir.Pkg.build_command = None
   ; install_command = None
   ; depends = []
+  ; depexts = []
   ; info =
-      { Lock_dir.Pkg_info.name; version; dev = false; source = None; extra_sources = [] }
+      { Lock_dir.Pkg_info.name
+      ; version
+      ; dev = false
+      ; avoid = false
+      ; source = None
+      ; extra_sources = []
+      }
   ; exported_env = []
   }
 ;;
@@ -169,10 +176,12 @@ let%expect_test "encode/decode round trip test for lockdir with simple deps" =
               { build_command = None
               ; install_command = None
               ; depends = []
+              ; depexts = []
               ; info =
                   { name = "bar"
                   ; version = "0.2.0"
                   ; dev = false
+                  ; avoid = false
                   ; source = None
                   ; extra_sources = []
                   }
@@ -182,10 +191,12 @@ let%expect_test "encode/decode round trip test for lockdir with simple deps" =
               { build_command = None
               ; install_command = None
               ; depends = []
+              ; depexts = []
               ; info =
                   { name = "foo"
                   ; version = "0.1.0"
                   ; dev = false
+                  ; avoid = false
                   ; source = None
                   ; extra_sources = []
                   }
@@ -198,7 +209,8 @@ let%expect_test "encode/decode round trip test for lockdir with simple deps" =
         { variable_values = [ ("os", "linux") ]
         ; unset_variables = [ "os-family" ]
         }
-    } |}]
+    }
+    |}]
 ;;
 
 let%expect_test "encode/decode round trip test for lockdir with complex deps" =
@@ -303,13 +315,15 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
     ; packages =
         map
           { "a" :
-              { build_command = Some Action [ "progn"; [ "echo"; "hello" ] ]
+              { build_command = Some (Action [ "progn"; [ "echo"; "hello" ] ])
               ; install_command = Some [ "system"; "echo 'world'" ]
               ; depends = []
+              ; depexts = []
               ; info =
                   { name = "a"
                   ; version = "0.1.0"
                   ; dev = false
+                  ; avoid = false
                   ; source = Some { url = "file:///tmp/a"; checksum = None }
                   ; extra_sources =
                       [ ("one", { url = "file:///tmp/a"; checksum = None })
@@ -322,10 +336,12 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
               { build_command = None
               ; install_command = None
               ; depends = [ ("complex_lock_dir/b.pkg:3", "a") ]
+              ; depexts = []
               ; info =
                   { name = "b"
                   ; version = "dev"
                   ; dev = true
+                  ; avoid = false
                   ; source =
                       Some
                         { url = "https://github.com/foo/b"
@@ -344,10 +360,12 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
                   [ ("complex_lock_dir/c.pkg:3", "a")
                   ; ("complex_lock_dir/c.pkg:3", "b")
                   ]
+              ; depexts = []
               ; info =
                   { name = "c"
                   ; version = "0.2"
                   ; dev = false
+                  ; avoid = false
                   ; source =
                       Some { url = "https://github.com/foo/c"; checksum = None }
                   ; extra_sources = []
@@ -362,7 +380,8 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
         }
     ; expanded_solver_variable_bindings =
         { variable_values = []; unset_variables = [] }
-    } |}]
+    }
+    |}]
 ;;
 
 let%expect_test "encode/decode round trip test with locked repo revision" =
@@ -413,10 +432,12 @@ let%expect_test "encode/decode round trip test with locked repo revision" =
               { build_command = None
               ; install_command = None
               ; depends = []
+              ; depexts = []
               ; info =
                   { name = "a"
                   ; version = "0.1.0"
                   ; dev = false
+                  ; avoid = false
                   ; source = None
                   ; extra_sources = []
                   }
@@ -426,10 +447,12 @@ let%expect_test "encode/decode round trip test with locked repo revision" =
               { build_command = None
               ; install_command = None
               ; depends = []
+              ; depexts = []
               ; info =
                   { name = "b"
                   ; version = "dev"
                   ; dev = false
+                  ; avoid = false
                   ; source = None
                   ; extra_sources = []
                   }
@@ -439,10 +462,12 @@ let%expect_test "encode/decode round trip test with locked repo revision" =
               { build_command = None
               ; install_command = None
               ; depends = []
+              ; depexts = []
               ; info =
                   { name = "c"
                   ; version = "0.2"
                   ; dev = false
+                  ; avoid = false
                   ; source = None
                   ; extra_sources = []
                   }
@@ -460,5 +485,6 @@ let%expect_test "encode/decode round trip test with locked repo revision" =
         }
     ; expanded_solver_variable_bindings =
         { variable_values = []; unset_variables = [] }
-    } |}]
+    }
+    |}]
 ;;
